package org.jeecg.modules.bytedance.report.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.jeecg.common.api.vo.Result;
import org.jeecg.modules.bytedance.advertise.entity.AiBytedanceAdvertiserStrategy;
import org.jeecg.modules.bytedance.advertise.entity.ByteDanceAdvertisePlan;
import org.jeecg.modules.bytedance.advertise.entity.RuleDataAccount;
import org.jeecg.modules.bytedance.advertise.mapper.AiBytedanceAdvertiserStrategyMapper;
import org.jeecg.modules.bytedance.advertise.mapper.ByteDanceAdvertisePlanMapper;
import org.jeecg.modules.bytedance.common.utils.Check;
import org.jeecg.modules.bytedance.report.entity.BytedanceAdvertiserHourlyReport;
import org.jeecg.modules.bytedance.report.entity.vo.ReportCostVo;
import org.jeecg.modules.bytedance.report.mapper.BytedanceAdvertiserHourlyReportMapper;
import org.jeecg.modules.bytedance.report.service.IBytedanceAdvertiserHourlyReportService;
import org.jeecg.modules.bytedance.report.utils.TimeStartAndEndUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @Description: 广告主小时级别报表信息
 * @Author: jeecg-boot
 * @Date: 2021-05-18
 * @Version: V1.0
 */
@Service
@Primary

public class BytedanceAdvertiserHourlyReportServiceImpl extends ServiceImpl<BytedanceAdvertiserHourlyReportMapper, BytedanceAdvertiserHourlyReport> implements IBytedanceAdvertiserHourlyReportService {


    @Resource
    private BytedanceAdvertiserHourlyReportMapper bytedanceAdvertiserHourlyReportMapper;

    @Resource
    private AiBytedanceAdvertiserStrategyMapper aiBytedanceAdvertiserStrategyMapper;

    @Resource
    private ByteDanceAdvertisePlanMapper byteDanceAdvertisePlanMapper;


    /**
     *
     * @description:
     *
     * @param accountId 账户id
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @return: org.jeecg.common.api.vo.Result
     * @author: zianY
     * @time: 2021/5/19 16:56
     */
    @Override
    public Result getReportYearOnYearCost(String accountId, String startTime, String endTime) throws Exception{
        DecimalFormat df = new DecimalFormat("0.00%");
        Map<String,Object> resultMap = new HashMap<>();
        double compare = 0;
        List<ReportCostVo> nowCostList = new ArrayList<>();
        List<ReportCostVo> lastCostList = new ArrayList<>();

        //当前时间段数据
        nowCostList = bytedanceAdvertiserHourlyReportMapper.getReportCostInfo(accountId,startTime,endTime);
        //当前阶段 总花费
        double nowCost = nowCostList.stream().mapToDouble(ReportCostVo::getCost).sum();
        resultMap.put("nowCost", nowCost);

        //根据本阶段时间 获取 上阶段 开始-截至 时间
        Map<String,String> lastTimeMap = TimeStartAndEndUtil.getStartEndTime(startTime,endTime);
        //上阶段数据
        lastCostList = bytedanceAdvertiserHourlyReportMapper.getReportCostInfo(accountId,lastTimeMap.get("lastTimeStart"),lastTimeMap.get("lastTimeEnd"));
        //上阶段花费
        double lastCost = lastCostList.stream().mapToDouble(ReportCostVo::getCost).sum();
        resultMap.put("lastCost", lastCost);

        //环比 =（当前花费 - 上阶段花费 ）/ 上阶段花费
        compare = (nowCost - lastCost) / lastCost;
        resultMap.put("compare", df.format(compare));

        //花费占比 = 花费 / 账户预算
        //获取账户预算
        QueryWrapper<AiBytedanceAdvertiserStrategy> strategyQueryWrapper = new QueryWrapper<>();
        strategyQueryWrapper.eq("account_id",accountId);
        strategyQueryWrapper.eq("status",0);
        strategyQueryWrapper.last("limit 1");
        AiBytedanceAdvertiserStrategy strategy = aiBytedanceAdvertiserStrategyMapper.selectOne(strategyQueryWrapper);
        //没有预算则使用上阶段花费
        double accountBudget = Check.isNull(strategy) ? lastCost : strategy.getAccountBudget().doubleValue();
        //花费占比
        double costShare = nowCost / accountBudget;
        resultMap.put("costShare", df.format(costShare));

        //获取 本阶段  转化成本；转化数；转化率；平均千次展现费用。数据展示
        Map nowMap = getCost(nowCostList);
        // 上阶段数据
        Map lastMap = getCost(lastCostList);

        /**
         * 日期相差不为 0 则 展示 时间段（每天数据总和）数据
         * 日期相差天数为0  则 展示 24小时 数据
         */
        if (!lastTimeMap.get("daysBetween").equals("0")){
            //阶段 数据
          nowCostList = bytedanceAdvertiserHourlyReportMapper.getReportCostInfoStage(accountId,startTime,endTime);
          lastCostList = bytedanceAdvertiserHourlyReportMapper.getReportCostInfoStage(accountId,lastTimeMap.get("lastTimeStart"),lastTimeMap.get("lastTimeEnd"));
        }
        //当前阶段 每个时间 的 转化成本；转化数；转化率；平均千次展现费用。数据展示
        nowMap.put("costList", getCostList(nowCostList));
        //上个阶段 每个时间 的 转化成本；转化数；转化率；平均千次展现费用。数据展示
        lastMap.put("costList", getCostList(lastCostList));

        resultMap.put("nowMap", nowMap);
        resultMap.put("lastMap", lastMap);
        return Result.successMsg("成功。",resultMap);
    }


    /**
     * 获取 数据 （总）
     * @param list
     * @return
     */
    public Map<String,Object> getCost(List<ReportCostVo> list){
        Map<String,Object> map = new HashMap<>();
        //花费
        double cost = list.stream().mapToDouble(ReportCostVo::getCost).sum();
        //转化成本 = 总花费 / 转化数
        int converNum = list.stream().mapToInt(ReportCostVo::getConvertNum).sum();
        double conver = cost / converNum;
        map.put("conver", String.format("%.2f", conver));
        map.put("converNum", converNum);
        //转化率 = 转化数 / 点击数 * 100
        int clickNum = list.stream().mapToInt(ReportCostVo::getClick).sum();
        double click = Double.valueOf(converNum) /  Double.valueOf(clickNum) * 100;
        map.put("converRate", String.format("%.2f", click));
        // 平均千次展显费用 = 总花费/展示数 * 1000
        int showNum = list.stream().mapToInt(ReportCostVo::getShowNum).sum();
        double show = (cost / showNum) * 1000 ;
        map.put("averageShow", String.format("%.2f", show));
        return map;
    }

    //获取 数据 （时间段）
    public List<ReportCostVo> getCostList(List<ReportCostVo> list){
        list.forEach(reportCostVo -> {
            //花费
            double cost = reportCostVo.getCost();
            //转化成本 = 总花费 / 转化数
            int converNum = reportCostVo.getConvertNum();
            double conver = cost / converNum;
            reportCostVo.setConver(String.format("%.2f", conver));
            reportCostVo.setConvertNum(converNum);

            //转化率 = 转化数 / 点击数 * 100
            int clickNum = reportCostVo.getClick();
            double click = Double.valueOf(converNum) /  Double.valueOf(clickNum) * 100;
            reportCostVo.setConverRate(String.format("%.2f", click));

            // 平均千次展显费用 = 总花费/展示数 * 1000
            int showNum = reportCostVo.getShowNum();
            double show = (cost / showNum) * 1000 ;
            reportCostVo.setAverageShow(String.format("%.2f", show));
        });
        return list;
    }



    /**
     *
     * @description:
     *
     * @param userId 当前登录用户id
     * @param type 类型 kuaishou bytedance
     * @return: org.jeecg.common.api.vo.Result
     * @author: zianY
     */
    @Override
    public Result initAccountIdAndTime(String userId, String type) throws Exception {
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
        Map<String,Object> resultMap = new HashMap<>();
        Map<String,Object> map = bytedanceAdvertiserHourlyReportMapper.selAcountIdCostMax(userId,type);
        resultMap.put("startTime", formatter.format(new Date()));
        resultMap.put("endTime", formatter.format(new Date()));
        resultMap.put("account", map);
        return Result.successMsg("成功",resultMap);
    }


    /**
     *
     * @description: 查询账户余额；花费
     *
     * @param accountId
     * @return: org.jeecg.common.api.vo.Result
     * @author: zianY
     */
    @Override
    public Result getAccountCost(String accountId) throws Exception {

        Map<String,Object> resultMap = new HashMap<>();
        Map<String,Object> accountMap = bytedanceAdvertiserHourlyReportMapper.selectAccountCost(accountId);
        //获取账户预算
        QueryWrapper<AiBytedanceAdvertiserStrategy> strategyQueryWrapper = new QueryWrapper<>();
        strategyQueryWrapper.eq("account_id",accountId);
        strategyQueryWrapper.eq("status",0);
        strategyQueryWrapper.last("limit 1");
        AiBytedanceAdvertiserStrategy strategy = aiBytedanceAdvertiserStrategyMapper.selectOne(strategyQueryWrapper);
        double accountBudget = Check.isNull(strategy) ? 0 : strategy.getAccountBudget().doubleValue();
        resultMap.put("accountMap", accountMap);
        resultMap.put("accountBudget", accountBudget);

        return Result.successMsg("成功。", resultMap);
    }



    /**
     *
     * @description:查询计划数据
     *
     * @param accountId 账户id
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @return: org.jeecg.common.api.vo.Result
     * @author: zianY
     */
    @Override
    public Result getReportPlanCost(String accountId, String startTime, String endTime,Integer pageNum,Integer pageSize) throws Exception {
        DecimalFormat df = new DecimalFormat("0.00%");
        //根据本阶段时间 获取 上阶段 开始-截至 时间
        Map<String,String> lastTimeMap = TimeStartAndEndUtil.getStartEndTime(startTime,endTime);
        PageHelper.startPage(pageNum,pageSize);
        //查询计划数据时报
       List<ReportCostVo> reportCostVoList = bytedanceAdvertiserHourlyReportMapper.selectReportPlanCostInfo(accountId, null, startTime,endTime);
       reportCostVoList.forEach(reportCostVo -> {
           //出价
           QueryWrapper<ByteDanceAdvertisePlan> planQueryWrapper = new QueryWrapper<>();
           planQueryWrapper.eq("account_id",accountId);
           planQueryWrapper.last("limit 1");
           ByteDanceAdvertisePlan planInfo = byteDanceAdvertisePlanMapper.selectOne(planQueryWrapper);
           reportCostVo.setCpaBid(planInfo.getCpaBid());
           reportCostVo.setBid(planInfo.getBid());

           // 平均千次展显费用 = 总花费/展示数 * 1000
           double show = 0;
           if (reportCostVo.getShowNum() != 0){
               show = (reportCostVo.getCost() / reportCostVo.getShowNum()) * 1000 ;
           }
           reportCostVo.setAverageShow(String.format("%.2f", show));

           double avgClick = 0;
           double clickRate = 0;
           double convetRate = 0;
           if (reportCostVo.getClick() != 0){
               //平均点击单价 = 花费 / 点击数
               avgClick = reportCostVo.getCost() / reportCostVo.getClick().doubleValue();
               //点击率 = 点击数 / 展示数
               clickRate = reportCostVo.getClick().doubleValue() / reportCostVo.getShowNum().doubleValue();
               //转化率 = 转化数 / 点击数 * 100
               convetRate = Double.valueOf(reportCostVo.getConvertNum()) /  Double.valueOf(reportCostVo.getClick()) * 100;
           }

           reportCostVo.setAverageClick(String.format("%.2f", avgClick));
           reportCostVo.setClickRate(df.format(clickRate));
           reportCostVo.setConverRate(String.format("%.2f", convetRate));

           //转化成本 = 总花费 / 转化数
           double conver = 0;
           if (reportCostVo.getConvertNum() != 0){
               conver = reportCostVo.getCost() / reportCostVo.getConvertNum();
           }
           reportCostVo.setConver(String.format("%.2f", conver));

           // 计划id 和 上阶段 时间 查询 上阶段数据
           List<ReportCostVo> lastReportCostVoList = bytedanceAdvertiserHourlyReportMapper.selectReportPlanCostInfo(accountId, reportCostVo.getAdId(), lastTimeMap.get("lastTimeStart"),lastTimeMap.get("lastTimeEnd"));
           double compare = 0;
           if (lastReportCostVoList.size() > 0){
               if (lastReportCostVoList.get(0).getCost() !=0){
                   //环比 =（当前花费 - 上阶段花费 ）/ 上阶段花费
                   compare = (reportCostVo.getCost() - lastReportCostVoList.get(0).getCost()) / lastReportCostVoList.get(0).getCost();
               }
           }
           reportCostVo.setCompare(df.format(compare));

           //操作状态
           QueryWrapper<ByteDanceAdvertisePlan> advertisePlanQueryWrapper = new QueryWrapper<>();
           advertisePlanQueryWrapper.eq("account_id",accountId);
           advertisePlanQueryWrapper.eq("id",reportCostVo.getAdId());
           advertisePlanQueryWrapper.last("limit 1");
           ByteDanceAdvertisePlan advertisePlan = byteDanceAdvertisePlanMapper.selectOne(advertisePlanQueryWrapper);
           reportCostVo.setOptStatus(advertisePlan.getOptStatus());
       });
        PageInfo<ReportCostVo> pageInfo = new PageInfo<ReportCostVo>(reportCostVoList);
        return Result.successMsg("成功。", pageInfo);
    }


}
